/**************************************************************/
/*                                                            */
/*   File name :  libspeplot.c                                */
/*                                                            */
/*      Version:   2007-11-13                                 */
/*                                                            */
/*   Copyright (c) 2007 T.Kondo/NICT All Right Reserved       */
/*                                                            */
/**************************************************************/
/*

2007-11-13
  divided from libspeplot.h

*/

#include "libspeplot.h"   

void tmpnameget(char *tmpdir, char *fpath)  // original is in libcor.c
// temporary unique name get  2003-07-27
// use mkstemp instead of tmpnam when linux or FreeBSD 2005-09-16
{
    int k;
#ifdef WIN32                                        // 2005-09-16
    char cbuf[L_tmpnam+1];
#else                                               //2005-09-16
    int fd;                                         //2005-09-16
#endif                                              //2005-09-16
    char fname[80],dir[80];
    // check tmdir is accompanied with '/' or '\'
    k=strlen(tmpdir);
    if(k > 0){
       if(tmpdir[k-1] != '/' && tmpdir[k-1] != '\\'){
          tmpdir[k]='/';
          tmpdir[k+1]='\0';
       }
    }
#ifdef WIN32                                        // 2005-09-16
    tmpnam(cbuf);                                
    parse_dir_and_name_fm_fpath(cbuf,dir,fname); 
#else                                               // 2005-09-16
    strcpy(fname,"XXXXXX");                         // 2005-09-16
    fd=mkstemp(fname);                              // 2005-09-16
         // temprary file will be created by "mkstemp"
    close(fd);  // release file descripter          // 2005-09-16
                // to avoid infinit increase of fd  // 2005-09-16
    k=remove(fname);  // delete temporary file      // 2005-09-16
#endif                                              // 2005-09-16
    strcpy(fpath,tmpdir);
    strcat(fpath,fname);
}

/******************************************/
/*    Plot with PGPLOT and GNUPLOT        */
/******************************************/

void plotspe(speana_param_t *st,
             float ymaxdb, float ymindb)
{
    int kplot, m, k;
    char labfsampl[10];
#ifdef USE_GNUPLOT   // using GNUPLOT
    char ftemp4ch[4][60];
#endif

    st->xdata = (float *)malloc(st->nfft*sizeof(float));

    /* sampling frequency in text for print */
    if(st->sfreq < 1000){
        sprintf(labfsampl,"%dkHz\0",st->sfreq);
    } else {
        sprintf(labfsampl,"%dMHz\0",st->sfreq/1000);
    }

    for(kplot=0; kplot<2; kplot++){
       if(st->pmode==1 && kplot==1) goto bypass;
       if(st->pmode==2 && kplot==0) goto bypass;
       if(kplot==0) {
           strcpy(st->g_device,st->g_ps_device);
       } else {
           strcpy(st->g_device,st->g_display);
       }

#ifdef USE_GNUPLOT   // using gnuplot
       st->gp= popen(GNUPLOT_EXE,"w");
       if(st->gp==NULL) goto exitend;
       if(kplot==0) {
          //fprintf(gp,"set terminal post eps color\n");
          fprintf(st->gp,"set terminal post landscape 12 color\n");
          fprintf(st->gp,"set output \"%s\"\n",st->ps_fname);
       }
       if(kplot==1) {
          fprintf(st->gp,"set terminal %s\n",st->g_display);
       }
       fflush(st->gp);
       fprintf(st->gp,"set multiplot\n");
       fprintf(st->gp,"set size 0.5,0.5\n");

#else                // using pgplot
       //if(kplot==0) {
       //    strcpy(pgdevice,"%s/PS",st->ps_fname);
       //} else {
       //    strcpy(pgdevice,st->g_device);
       //}
       if(cpgopen(st->g_device) < 1) goto exitend;
       if(strncmp(st->g_device,"/GW",3)==0) cpgeras();   // erase just in case 2007-10-26
       if(strncmp(st->g_device,"/XWIN",5)==0 ||
           strncmp(st->g_device,"/XSER",5)==0 ) { /* set background white 
                                                   and text black */
           cpgscr(0,1.0,1.0,1.0);  /* Background color */
           cpgscr(1,0.0,0.0,0.0);  /* Foreground color */
       }

       cpgsubp(2,2);  // divide plot pannel into 2X2
#endif
       for(m=0; m<st->numch; m++){

#ifdef USE_GNUPLOT   // using gnuplot
           switch(m){
              case 0: fprintf(st->gp,"set origin 0.0,0.5\n"); break;
              case 1: fprintf(st->gp,"set origin 0.5,0.5\n"); break;
              case 2: fprintf(st->gp,"set origin 0.0,0.0\n"); break;
              case 3: fprintf(st->gp,"set origin 0.5,0.0\n"); break;
           }
           //fprintf(gp,"clear\n");   

#endif

           sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, Tavg(sec)=%.1f\0",
               m+1,st->adbit,labfsampl,st->sekibun);
           for(k=0; k<st->nfft; k++) {
                st->xdata[k]=st->spows[k*4+m];
           }

           if(st->amode > 0) {
                st->mode=1;
                plotautocor(st);
           } else {
                plotspe_a(st,ymaxdb,ymindb);
           }
#ifdef USE_GNUPLOT   // using gnuplot
           strcpy(ftemp4ch[m],st->ftemp);
#endif

       }

#ifdef USE_GNUPLOT   // using gnuplot
       fprintf(st->gp,"unset multiplot\n");
       fflush(st->gp);
  #ifdef WIN32
       if(kplot==1) getchar();  // this is necessary to keep display otherwise deleted
  #endif
       pclose(st->gp);
       for(m=0; m<st->numch; m++) remove(ftemp4ch[m]);

#else                // using pgplot
       /* panel undivided to date write */
       cpgsubp(1,1);
       /* date etc write */
       cpgiden();
       /* close the graphics device */
       if(strncmp(st->g_device,"/GW",3)!=0) cpgclos();   //2007-10-26
#endif
       if(kplot==0){
           printf("Created PostScript file ==> %s\n",st->ps_fname);
       }
 bypass:;
    }
exitend:;
    free(st->xdata);
}

void plotspe2(speana_param_t *st)
{
    /* plot spectrum (or auto cor) POSTSCRIPT OUT and XTERM*/
    int kplot, m, k;
    char labfsampl[10];
    float pmaxdat[4],fdatmax[4];
    int nmax=4;
#ifdef USE_GNUPLOT   // using gnuplot
    char ftemp4ch[4][60];
#endif

    st->xdata = (float *)malloc(st->nfft*sizeof(float));

    /* sampling frequency in text for print */
    if(st->sfreq < 1000){
        sprintf(labfsampl,"%dkHz\0",st->sfreq);
    } else {
        sprintf(labfsampl,"%dMHz\0",st->sfreq/1000);
    }

    for(kplot=0; kplot<2; kplot++){
        if(st->pmode==1 && kplot==1) goto bypass;
        if(st->pmode==2 && kplot==0) goto bypass;
       if(kplot==0) {
           strcpy(st->g_device,st->g_ps_device);
       } else {
           strcpy(st->g_device,st->g_display);
       }
#ifdef USE_GNUPLOT   // using gnuplot
       st->gp= popen(GNUPLOT_EXE,"w");
       if(st->gp==NULL) goto exitend;
       if(kplot==0) {
          //fprintf(gp,"set terminal post eps color\n");
          fprintf(st->gp,"set terminal post landscape 12 color\n");
          fprintf(st->gp,"set output \"%s\"\n",st->ps_fname);
       }
       if(kplot==1) {
          fprintf(st->gp,"set terminal %s\n",st->g_device);
       }
       fflush(st->gp);
       fprintf(st->gp,"set multiplot\n");
       fprintf(st->gp,"set size 0.5,0.5\n");
#else                // using pgplot
        //if(kplot==0) {
        //    strcpy(pgdevice,"/PS");
        //} else {
        //    strcpy(pgdevice,st->g_device);
        //}
        if(cpgopen(st->g_device) < 1) goto exitend;
        if(strncmp(st->g_device,"/GW",3)==0) cpgeras();   // erase just in case 2007-10-26
        if(strncmp(st->g_device,"/XWIN",5)==0 ||
           strncmp(st->g_device,"/XSER",5)==0 ) { /* set background white 
                                                   and text black */
           cpgscr(0,1.0,1.0,1.0);  /* Background color */
           cpgscr(1,0.0,0.0,0.0);  /* Foreground color */
        }

        cpgsubp(2,2);  // divide plot pannel into 2X2
#endif
        for(m=0; m<st->numch; m++){

#ifdef USE_GNUPLOT   // using gnuplot
           switch(m){
              case 0: fprintf(st->gp,"set origin 0.0,0.5\n"); break;
              case 1: fprintf(st->gp,"set origin 0.5,0.5\n"); break;
              case 2: fprintf(st->gp,"set origin 0.0,0.0\n"); break;
              case 3: fprintf(st->gp,"set origin 0.5,0.0\n"); break;
           }

#endif
           sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, Tavg(sec)=%.1f\0",
               m+1,st->adbit,labfsampl,st->sekibun);
                
           for(k=0; k<st->nfft; k++) st->xdata[k]=st->spows[k*4+m];

           if(st->amode > 0) {
              st->mode=1;
              plotautocor(st);
           } else {
              plotspe_a2(st,  pmaxdat, fdatmax, nmax);
              if(kplot==0){
                 printf("1st Max %fdbm at %fkHz\n",pmaxdat[0],fdatmax[0]);
                 printf("2nd Max %fdbm at %fkHz\n",pmaxdat[1],fdatmax[1]);
                 printf("3rd Max %fdbm at %fkHz\n",pmaxdat[2],fdatmax[2]);
                 printf("4th Max %fdbm at %fkHz\n",pmaxdat[3],fdatmax[3]);
              }
           }
#ifdef USE_GNUPLOT   // using gnuplot
           strcpy(ftemp4ch[m],st->ftemp);
#endif
        }

#ifdef USE_GNUPLOT   // using gnuplot
       fprintf(st->gp,"unset multiplot\n");
       fflush(st->gp);
  #ifdef WIN32
       if(kplot==1) getchar();  // this is necessary to keep display otherwise deleted
  #endif
       pclose(st->gp);
       for(m=0; m<st->numch; m++) remove(ftemp4ch[m]);
#else                // using pgplot
        /* panel undivided to date write */
       cpgsubp(1,1);
       /* date etc write */
       cpgiden();
       /* close the graphics device */
       if(strncmp(st->g_device,"/GW",3)!=0) cpgclos();   //2007-10-26
#endif
       if(kplot==0){
           printf("Created PostScript file ==> %s\n",st->ps_fname);
       }
    bypass:;
    }

exitend:;
    free(st->xdata);
}


void plotspe_a(speana_param_t *st, float ymaxdb, float ymindb)
/*
               mode2 -- YX: axis scale mode
                          X: frequency axis mode
                             =0: log scale  =1: linear scale
                          Y: amplitude axis mode
                             =0: log scale 
                             =1: linear scale (common for 4 channels)
                                  (use ymaxdb, ymindb)
                             =2: linear scale (each max and min)
*/
{
      float fmax, fmin, fsampl, df, ymax, ymin, pmax, pmin;
      float fscale;
      float *fdat, *powdat;
      char htitl[60], xtit[60], ytit[60];
      int n, npmax;
      int mode,ymode;

      ymode=st->mode2/10;
      mode=st->mode2 % 10;
      strcpy(htitl,"POWER SPECTRUM");



      /* frequency axis generation */

      //npmax=npos/2;
      npmax=st->nfft/2;

      fdat   = (float *)malloc(npmax*sizeof(float));
      powdat = (float *)malloc(npmax*sizeof(float));

      /* frequency axis */
      fsampl=(float)1000.0*(float)st->sfreq;
      df=fsampl/(float)st->nfft;
      if(st->sfreq > 8000){  // if sampling frequency exceeds 8 MHz
          fscale = 1.0e6;  // scaling in MHz
          strcpy(xtit,"FREQUENCY (MHz)");
      } else {
          fscale = 1000.0; // scaling in kHz
          strcpy(xtit,"FREQUENCY (kHz)");
      }
      if(mode == 0) {    // log scale
#ifdef USE_GNUPLOT   // using gnuplot
         fmax=(float)(fsampl/(2.0*fscale));
         fmin=(float)(df/fscale);
         for(n=0; n<npmax; n++)fdat[n]=(float)((float)n*df/fscale); 
#else // use PGPLOT
         fmax=(float)log10(fsampl/(2.0*fscale));
         fmin=(float)log10(df/fscale);
         for(n=0; n<npmax; n++) fdat[n]=(float)log10((float)n*df/fscale);
#endif
      } else {          //  linear scale
         fmax=(float)(fsampl/(2.0*fscale));
         fmin=0.0;
         for(n=0; n<npmax; n++){fdat[n]=(float)((float)n*df/fscale); }
      }

      /* amplitude axis */

      /* data copy, if necessary convert to linear scale */
      if(ymode == 0){    // log scale
         for(n=0; n<npmax; n++) powdat[n]=st->xdata[n];
      } else {   // linear scale
         /* convert to linear scale */
         for(n=0; n<npmax; n++) powdat[n]=(float)pow(10.0,st->xdata[n]/10.0);
      }

      /* data max, min check */
      for(n=0; n<npmax; n++){
         //powdat[n]=spow[n];
         if(n==0){
            pmax=powdat[n];
            pmin=powdat[n];
         } else {
            if(powdat[n] > pmax) pmax=powdat[n];
            if(powdat[n] < pmin) pmin=powdat[n];
         }
      }

      if(ymode == 0){    // log scale
         strcpy(ytit,"POWER (dBm)");
         if( st->adbit==1 ) strcpy(ytit,"POWER (dB)"); 
         ymax=10.0;
         ymin=-80.0;
         if (st->adbit == 8 ) ymin=-100;     // extend dynamic range 
      } else {   // linear scale
         strcpy(ytit,"POWER (linear)");
         ymax=pmax;
         ymin=pmin;
         if(ymode==1){
            ymax=(float)pow(10.0,ymaxdb/10.0);
            ymin=(float)pow(10.0,ymindb/10.0);
         }
      }

#ifdef USE_GNUPLOT   // using gnuplot

      /* define coordinate range of graph */
      if(mode == 0){   
           fprintf(st->gp,"set logscale x\n");
      } else {         
           fprintf(st->gp,"unset logscale\n");
      }

      fprintf(st->gp,"set xrange [%g:%g]\n",fmin,fmax);
      fprintf(st->gp,"set yrange [%g:%g]\n",ymin,ymax);
      
      /* label the axes */
      fprintf(st->gp,"set xlabel \"%s\"\n",xtit);
      fprintf(st->gp,"set ylabel \"%s\"\n",ytit);
      //fprintf(st->gp,"set title \"%s\"\n",htitl);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      if(strncmp(st->g_device,"windows",7)==0 || 
          strncmp(st->g_device,"x11",3)==0 ){ // display
         fprintf(st->gp,"set title \"%s\"\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      } else { // postscript
         fprintf(st->gp,"set title \" \"\n");  // this is to keep dummy space
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.08 center font \"Helvetica,10\"\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.03 center font \"Helvetica,10\"\n",st->lab2);
      }

      /* check data */
      if (pmax == pmin){
          fprintf(st->gp,"set label \"NO TIME VARIABLE\" at graph 0.5, 0.52 center\n");
          fprintf(st->gp,"set label \"COMPONENT FOUND\" at graph 0.5, 0.48 center\n");
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  
          gnline2(st->gp,2,fdat,powdat,st->ftemp); 
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          gnline2(st->gp,npmax,fdat,powdat,st->ftemp); 

      }
      fprintf(st->gp,"unset label\n");  // this is necessary otherwise 
                                    // CH2,3,4 label are overlapped 

#else  // using PGPLOT

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      if(mode == 0){   
         cpgenv(fmin, fmax, ymin, ymax, 0, 10);   // log scale
      } else {         
         cpgenv(fmin, fmax, ymin, ymax, 0, 0);   // linear scale
      }
      
      /* label the axes */
      cpglab(xtit,ytit,htitl);

      /* sub label write*/
      cpgsch(1.5);
      cpgmtxt("T",1.5,0.5,0.5,st->lab1);
      cpgmtxt("T",0.5,0.5,0.5,st->lab2);
      cpgsch(2.0);

      /* check data */
      if (pmax == pmin){
          cpgmtxt("B",-6.0,0.5,0.5,"NO TIME VARIABLE");
          cpgmtxt("B",-4.0,0.5,0.5,"COMPONENT FOUND");
          /* label the axes */
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          //cpgslw(1);    // set line width
          cpgline(npmax,fdat,powdat); 

      }
   
#endif

     free(powdat);
     free(fdat);
}

void plotspe_a2(speana_param_t *st,float *pmaxdat, 
               float *fdatmax, int nmax)
/*
        st->mode = 0; log frequency scale   1; linear frequency scale
*/
{
      float fmax, fmin, fsampl, df, ymax, ymin, pmax, pmin, maxdbm, mindbm;
      float fscale;
      float *fdat, *powdat;
      float fatmax;
      char htitl[60], xtit[60], ytit[60];
      int n, npmax,k, ncount,nmaxpos;
      //float pmaxdat[4],fdatmax[4];
      //nmax=4;
      strcpy(htitl,"POWER SPECTRUM");

      /* frequency axis generation */

      npmax=st->nfft/2;

      fdat   = (float *)malloc(npmax*sizeof(float));
      powdat = (float *)malloc(npmax*sizeof(float));

      fsampl=(float)(1000.0*(float)st->sfreq);
      df=fsampl/(float)st->nfft;
      if(st->sfreq > 8000){  // if sampling frequency exceeds 8 MHz
          fscale = 1.0e6;  // scaling in MHz
          strcpy(xtit,"FREQUENCY (MHz)");
      } else {
          fscale = 1000.0; // scaling in kHz
          strcpy(xtit,"FREQUENCY (kHz)");
      }
      if(st->mode == 0) {    // log scale
         fmax=(float)log10(fsampl/(2.0*fscale));
         fmin=(float)log10(df/fscale);
         for(n=0; n<npmax; n++){fdat[n]=(float)log10((float)n*df/fscale); }
      } else {          //  linear scale
         fmax=(float)(fsampl/(2.0*fscale));
         fmin=0.0;
         //fmaxkhz=990.0;
         //fminkhz=920.0;
         if(st->f1khz != 0.0) fmin=(float)(st->f1khz/(fscale/1000.0));
         if(st->f2khz != 0.0) fmax=(float)(st->f2khz/(fscale/1000.0));
         for(n=0; n<npmax; n++) fdat[n]=(float)(n*df/fscale); 
      }

      strcpy(ytit,"POWER (dBm)");
      if( st->adbit==1 ) strcpy(ytit,"POWER (dB)"); 
      ymax=10.0;
      ymin=-80.0;
      //ymax=-34.0;
      //ymin=-40.0;
      if (st->adbit == 8 ){ ymin=-100; }    // extend dynamic range 

      /* data max, min check */
      //printf("max search start %d\n",npmax);
      
      for(n=0; n<npmax; n++) powdat[n]=st->xdata[n];
    
      pmin= (float)1.0e30;
      for(ncount=0; ncount<nmax; ncount++){
          pmax=(float)(-1.0e30);
          k=0;
          for(n=0; n<npmax; n++){
            if(fdat[n]>=fmin && fdat[n]<=fmax){
               if(powdat[n] > pmax){ 
                   pmax=powdat[n];
                   fatmax=fdat[n];
                   pmaxdat[ncount]=pmax;
                   fdatmax[ncount]=(float)(fatmax*(fscale/1000.0));  // to obtain in kHz
                   nmaxpos=n;
               }
               if(ncount==0) {
                    if(powdat[n] < pmin) pmin=powdat[n]; 
               }
               k++;
            }
          }
         powdat[nmaxpos]=(float)(-10.0e30);
         //printf("pmax,fatmax, i %f %f %d\n",pmax,fatmax,i);
      }
     /*
      printf("Number of points = %d\n",k);
      printf("1st Max %fdbm at %fkHz\n",pmaxdat[0],fdatmax[0]);
      printf("2nd Max %fdbm at %fkHz\n",pmaxdat[1],fdatmax[1]);
      printf("3rd Max %fdbm at %fkHz\n",pmaxdat[2],fdatmax[2]);
      printf("4th Max %fdbm at %fkHz\n",pmaxdat[3],fdatmax[3]);
     */
      maxdbm=pmaxdat[0];
      mindbm=pmin;
      ymax=pmaxdat[0];
      ymin=pmin;
      //printf("ymax,ymin %f %f\n",ymax,ymin);
    
      for(n=0; n<npmax; n++) powdat[n]=st->xdata[n];

#ifdef USE_GNUPLOT   // using gnuplot

      /* define coordinate range of graph */
      if(st->mode == 0){   
         //cpgenv(fmin, fmax, ymin, ymax, 0, 10);   // log scale
          fprintf(st->gp,"set logscale x\n");
      } else {         
         //cpgenv(fmin, fmax, ymin, ymax, 0, 0);   // linear scale
          fprintf(st->gp,"unset logscale\n");
      }
      fprintf(st->gp,"set xrange [%g:%g]\n",fmin,fmax);
      fprintf(st->gp,"set yrange [%g:%g]\n",ymin,ymax);
      
      /* label the axes */
      //cpglab(xtit,ytit,htitl);
      //fprintf(st->gp,"set title \"%s\"\n",htitl);
      fprintf(st->gp,"set xlabel \"%s\"\n",xtit);
      fprintf(st->gp,"set ylabel \"%s\"\n",ytit);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      if(strncmp(st->g_device,"windows",7)==0 || 
          strncmp(st->g_device,"x11",3)==0 ){ // display
         fprintf(st->gp,"set title \"%s\"\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      } else { // postscript
         fprintf(st->gp,"set title \" \"\n");  // this is to keep dummy space
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.08 center font \"Helvetica,10\"\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.03 center font \"Helvetica,10\"\n",st->lab2);
      }

      /* check data */
      if (pmax == pmin){
          fprintf(st->gp,"set label \"NO TIME VARIABLE\" at graph 0.5, 0.52 center\n");
          fprintf(st->gp,"set label \"COMPONENT FOUND\" at graph 0.5, 0.48 center\n");
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  
          gnline2(st->gp,2,fdat,powdat,st->ftemp); 
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          gnline2(st->gp,npmax,fdat,powdat,st->ftemp); 

      }
      fprintf(st->gp,"unset label\n");  // this is necessary otherwise 
                                    // CH2,3,4 label are overlapped 

#else   // use PGPLOT

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      if(st->mode == 0){   
         cpgenv(fmin, fmax, ymin, ymax, 0, 10);   // log scale
      } else {         
         cpgenv(fmin, fmax, ymin, ymax, 0, 0);   // linear scale
      }
      
      /* label the axes */
       cpglab(xtit,ytit,htitl);

      /* sub label write*/
       cpgsch(1.5);
       cpgmtxt("T",1.5,0.5,0.5,st->lab1);
       cpgmtxt("T",0.5,0.5,0.5,st->lab2);
       cpgsch(2.0);

      /* check data */
      if (pmax == pmin){
          cpgmtxt("B",-6.0,0.5,0.5,"NO TIME VARIABLE");
          cpgmtxt("B",-4.0,0.5,0.5,"COMPONENT FOUND");
          /* label the axes */
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          //cpgslw(1);    // set line width
          cpgline(npmax,fdat,powdat); 

      }

#endif

     free(powdat);
     free(fdat);
}


void plotautocor(speana_param_t *st)
/*
        st->mode = 0; log frequency scale   1; linear frequency scale
*/
{
      //float cofst,tmax, tmin, fsampl, dt, ymax, ymin, pmax, pmin, maxdbm, mindbm;
      float tmax, tmin, fsampl, dt, ymax, ymin, pmax, pmin;
      float *tdat, *pdat;
      //float x,y;
      char htitl[60], xtit[60], ytit[60];
      int n, k, npmax, nhalf;

      strcpy(htitl,"AUTO CORRELATION (ABS)");

      /* time axis generation */

      if(st->amode <= 1) {
         npmax=st->nfft;
      } else {
         npmax=st->amode;
      }
      tdat = (float *)malloc(npmax*sizeof(float));
      pdat = (float *)malloc(npmax*sizeof(float));
      nhalf=npmax/2;

      fsampl=(float)(1000.0*(float)st->sfreq);
      dt=(float)(1.0/fsampl);
      for(n=0; n<npmax; n++){
          tdat[n]=(float)((float)(n-nhalf)*dt*1.0e6);   // usec unit
      }
      tmax=tdat[npmax-1];
      tmin=tdat[0];
      //printf("Debug %d %f %f\n",st->sfreq,tmax,tmin);

      strcpy(xtit,"DELAY TIME (us)");
      strcpy(ytit,"CORRELATION AMP");
      ymax=(float)1.1;
      ymin=0.0;

      /* data range limited copy and max, min check */
     
      for(n=0; n<npmax; n++){
         k=n-nhalf;
         if(k < 0) k=st->nfft+k;
         pdat[n]=st->xdata[k];
         if(pdat[n]<0) pdat[n]=-pdat[n];  // to take absolute value
         if(n==0){
            pmax=pdat[n];
            pmin=pdat[n];
         } else {
            if(pdat[n] > pmax) pmax=pdat[n];
            if(pdat[n] < pmin) pmin=pdat[n];
         }
      }

#ifdef USE_GNUPLOT   // using gnuplot
      

      /* define coordinate range of graph */
      if(st->mode == 0){   
           fprintf(st->gp,"set logscale x\n");
      } else {         
           fprintf(st->gp,"unset logscale\n");
      }
      fprintf(st->gp,"set xrange [%g:%g]\n",tmin,tmax);
      fprintf(st->gp,"set yrange [%g:%g]\n",ymin,ymax);
      
      /* label the axes */
      //fprintf(st->gp,"set title \"%s\"\n",htitl);
      fprintf(st->gp,"set xlabel \"%s\"\n",xtit);
      fprintf(st->gp,"set ylabel \"%s\"\n",ytit);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.13 center\n",st->lab1);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      if(strncmp(st->g_device,"windows",7)==0 || 
          strncmp(st->g_device,"x11",3)==0 ){ // display
         fprintf(st->gp,"set title \"%s\"\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      } else { // postscript
         fprintf(st->gp,"set title \" \"\n");  // this is to keep dummy space
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.08 center font \"Helvetica,10\"\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.03 center font \"Helvetica,10\"\n",st->lab2);
      }

      /* check data */
      if (pmax == pmin){
          fprintf(st->gp,"set label \"NO DATA FOR PLOT\" at graph 0.5, 0.5 center\n");
          tdat[0]=tdat[1];
          pdat[0]=pdat[1];  
          gnline2(st->gp,2,tdat,pdat,st->ftemp); // dummy plot to get coordinate box
          
      } else {
      
        /* plot the line graph */
         gnline2(st->gp,npmax,tdat,pdat,st->ftemp); 
      }
      fprintf(st->gp,"unset label\n");  // this is necessary otherwise 
                                    // CH2,3,4 label are overlapped 

#else  // using PGPLOT

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      if(st->mode == 0){   
         cpgenv(tmin, tmax, ymin, ymax, 0, 10);   // log scale
      } else {         
         cpgenv(tmin, tmax, ymin, ymax, 0, 0);   // linear scale
      }
      
      /* label the axes */
      cpglab(xtit,ytit,htitl);

      /* sub label write*/
      cpgsch(1.5);
      cpgmtxt("T",1.5,0.5,0.5,st->lab1);
      cpgmtxt("T",0.5,0.5,0.5,st->lab2);
      cpgsch(2.0);

      /* check data */
      if (pmax == pmin){
          cpgmtxt("B",-6.0,0.5,0.5,"NO DATA FOR PLOT");
          
      } else {

          cpgline(npmax,tdat,pdat); 

      }

#endif

     free(pdat);
     free(tdat);
}


/******************************************/
/*    Plot with GNUPLOT                   */
/******************************************/

void gnline(FILE *gp,  int ndata,float *xdata, float *ydata)
{
    /* donot use this function for multiplot
      use gnline2 for multi plot
    */
    int n;
    char *ftemp="gnplot.tmp"; 
    FILE *fo;

    if( (fo  = fopen( ftemp,"w" )) == NULL ){
        printf(" File (%s) Open Error! \n",ftemp);
        goto exit1;
    } else {
        //printf(" File (%s) Opened for GNUPLOT data write! \n",ftemp);
        for(n=0; n<ndata; n++){
            fprintf(fo, "%g %g\n",xdata[n],ydata[n]);
        }
    }
    //fflush(fo);
    fclose(fo);
    //tk_sleep(10);  // suspend 1 msec 
    fprintf(gp,"plot \"%s\" with lines notitle lt -1 lw 0.5\n",ftemp);
    fflush(gp);
exit1:;
}

void gnline2(FILE *gp,int ndata,float *xdata, float *ydata, char *ftemp)
{
 /*
   [in]
      gp  --- pointer to popened stream
      ndata --  number of points in xdata and ydata
      xdata[ndata] --  X data
      ydata[ndata] --  Y data
   [return]
      ftemp        --  file name used for line data to interface to GNUPLOT
                       should be deleted after pclose(gp)
*/

    int n;
    FILE *fo;
#ifdef WIN32
    char ftemptab[][20]={"gnuplot1.tmp","gnuplot2.tmp","gnuplot3.tmp","gnuplot4.tmp"};
    strcpy(ftemp,ftemptab[tempcounter]);
    tempcounter++;        // tempcounter is a gloval variable
    if(tempcounter >= 4) tempcounter=0;
#else
    tmpnameget("./", ftemp);  // original is in libcor.c
#endif
    if( (fo  = fopen( ftemp,"w" )) == NULL ){
        printf(" File (%s) Open Error! \n",ftemp);
        goto exit1;
    } else {
        //printf(" File (%s) Opened for GNUPLOT data write! \n",ftemp);
        for(n=0; n<ndata; n++){
            fprintf(fo, "%g %g\n",xdata[n],ydata[n]);
        }
    }
    fclose(fo);
    fprintf(gp,"plot \"%s\" with lines notitle lt -1 lw 0.5\n",ftemp);
    fflush(gp);
    //remove(ftemp);  //should be deleted after pclose(gp)
exit1:;

}
